module ParameterValidation

  class Rule
    attr_reader :type, :optional, :formatter
    def initialize(type, optional=false, formatter=nil)
      @type = type
      @optional = optional
      @formatter = formatter
    end
  end
  
  def self.validate(params, rules)
    error_messages = []
    params.each do |name, value|
      error_messages << "Unknown parameter: [ #{name} ]" unless rules.has_key? name
    end
    rules.each do |name, rule|
      param = params[name]
      if param
        if param.is_a? rule.type
          if rule.formatter
            begin
              params[name] = rule.formatter.call(param)
            rescue Exception => e
              error_messages << "Error formatting [ #{name} ]: " + e.message
            end
          end
        else
          error_messages << "Parameter [ #{name} ] must be of type #{rule.type}."
        end
      elsif !rule.optional
        error_messages << "Missing required parameter: [ #{name} ]"
      end
    end
    error_messages
  end

  def self.keys_to_symbols(hash)
    output = {}
    hash.each { |k,v| output[k.to_sym] = v }
    output
  end

  def self.fact_formatter(hash)
    if hash['name'] && !hash['name'].empty?
      return { hash['name'] => case_insensitive(hash['value']) }
    end
    return nil
  end

  def self.case_insensitive(string)
    "/(?i:#{string})/"
  end

end
